/***************************************************************************
 *   Copyright (C) 2007 by David Bitseff                                   *
 *   dbitsef@zipcon.net                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "varstablewidget.h"
#include "logger.h"

VarsTableWidget::VarsTableWidget(QWidget* parent)
{
	VarsTableWidget(0, 0, parent);
	activeColor   = QColor(200,230,240);
	inactiveColor = Qt::white;
	step = 0.1;
	start_item = 0;
}

VarsTableWidget::VarsTableWidget(int rows, int cols, QWidget* parent)
	: QTableWidget(rows, cols, parent)
{
}

void VarsTableWidget::wheelEvent(QWheelEvent* e)
{
	QTableWidgetItem* item = itemAt(e->pos());
	double item_data = QString(item->text()).toDouble();
	double nstep = step;
	if (e->modifiers() & Qt::ShiftModifier)
		nstep *= 0.10;
	else if (e->modifiers() & Qt::ControlModifier)
		nstep *= 10.0;

	if (e->delta() < 0)
		nstep *= -1.0;
	double inc_value = item_data + nstep;
	item->setText(QString::number(inc_value, 'f', VARS_PRECISION));
	colorItem(item, inc_value);
}

void VarsTableWidget::keyPressEvent(QKeyEvent* e)
{
	switch (e->key())
	{
		case Qt::Key_BracketLeft:
			step *= 10.0;
			break;

		case Qt::Key_BracketRight:
			step *= 0.10;
			break;

		case Qt::Key_Return:
			emit valueUpdated();
			break;

		default:
			QTableWidget::keyPressEvent(e);
	}
}

void VarsTableWidget::mousePressEvent(QMouseEvent* e)
{
	if (e->button() == Qt::LeftButton)
	{
		start_item = itemAt(e->pos());
		last_pos = e->posF();
		start_value = start_item->text().toDouble();
		QTableWidget::mousePressEvent(e);
		e->accept();
	}
}

void VarsTableWidget::mouseMoveEvent(QMouseEvent* e)
{
	if ((e->buttons() & Qt::LeftButton) && start_item)
	{
		double item_data = QString(start_item->text()).toDouble();
		double nstep = step;
		if (e->modifiers() & Qt::ShiftModifier)
			nstep *= 0.10;
		else if (e->modifiers() & Qt::ControlModifier)
			nstep *= 10.0;

		double dy = e->y() - last_pos.y();
		last_pos = e->posF();
		if (dy == 0.0) return;
		if (dy > 0)
			nstep *= -1.0;
		double inc_value = item_data + nstep;
		start_item->setText(QString::number(inc_value, 'f', VARS_PRECISION));
		colorItem(start_item, inc_value);

		e->accept();
		emit valueUpdated();
	}
}

void VarsTableWidget::mouseReleaseEvent(QMouseEvent* e)
{
	if ((e->button() == Qt::LeftButton)
		&& start_item
		&& start_value != start_item->text().toDouble())
	{
		start_item = 0;
		e->accept();
		emit undoStateSignal();
	}
}

void VarsTableWidget::colorItem(QTableWidgetItem* item, double value)
{
	if (value != 0.0)
		item->setBackground(activeColor);
	else
		item->setBackground(inactiveColor);
}
